searchData={"items":[{"type":"module","title":"PowerOfThree","doc":"The `PowerOfThree` defines three macros to be used with Ecto.Schema to creates cube config files.\nThe PowerOfThree must be used after `using Ecto.Schema `.\nThe `Ecto.Schema` defines table column names to be used in measure and dimensions defenitions.\n\nThe definition of the Cude is possible through main APIs:\n`cube/3`.\n\n`cube/3` has to define `sql_table:` that is refering Ecto schema `source`.\n\nAfter using `Ecto.Schema` and `PowerOfThree` define cube with `cube/2` macro.","ref":"PowerOfThree.html"},{"type":"module","title":"Example - PowerOfThree","doc":"defmodule Example.Customer do\n      use Ecto.Schema\n      use PowerOfThree\n\n      schema \"customer\" do\n        field(:first_name, :string)\n        field(:last_name, :string)\n        field(:email, :string)\n        field(:birthday_day, :integer)\n        field(:birthday_month, :integer)\n        field(:brand_code, :string)\n        field(:market_code, :string)\n      end\n\n      cube :of_customers,       # name of the cube: mandatory\n        sql_table: \"customer\",  # Ecto.Schema `source`: mandatory\n                                # Only `sql_table:` is supported. Must reference EctoSchema `:source`\n                                # the `sql:` is not supported and never will be. \n        description: \"of Customers\"\n                                # path through options in accordance with Cube DSL\n\n        dimension(\n          [:brand_code, :market_code, :email],\n                                # several fields of `customer` Ecto.Schema: mandatory\n                                # the list is validated against list of fields of EctoSchema\n          name: :email_per_brand_per_market,\n                                # dimensions `name:`, optional.\n          primary_key: true     # This `customer:` table supports only one unique combination of\n                                # `:brand_code`, `:market_code`, `:email`\n          )\n\n        dimension(\n          :first_name,          # a field of `customer` Ecto.Schema: mandatory\n                                # validated against list of fields of EctoSchema\n          name: :given_name,    # dimension `name:` optional\n          description: \"Given Name\"\n                                # path through options in accordance with Dimension DSL\n          )\n\n        measure(:count)         # measure of type `count:` is a special one: no column reference in `sql:` is needed\n                                # `name:` defaults to `count:`\n\n        measure(:email,         # measures counts distinct of `email:` column\n          name: :aquari         # given a `name:` and `description:` and `filter:` in options\n          type: :count_distinct,\n          description: \"Only count one zodiak sign\",\n          filters: [%{sql: \"(birthday_month = 1 AND birthday_day >= 20) OR (birthday_month = 2 AND birthday_day <= 18)\"}]\n                                # better be correct SQL refrencing correct columns - not validated now\n                                # `filter:` uses SQL clause to not count other categories of customers\n        )\n      end\n    end\n\nAfter creating a few dimensions and measures run `mix compile`. The following yaml is created for the above:\n\n```yaml\n\n---\ncubes:\n  - name: of_customers\n    description: of Customers\n    sql_table: customer\n    measures:\n      - name: count\n        type: count\n      - meta:\n          ecto_field: email\n          ecto_type: string\n        name: aquari\n        type: count_distinct\n        description: Only count one zodiak sign\n        filters:\n          - sql: (birthday_month = 1 AND birthday_day >= 20) OR (birthday_month = 2 AND birthday_day <= 18)\n        sql: email\n    dimensions:\n      - meta:\n          ecto_fields:\n            - brand_code\n            - market_code\n            - email\n        name: email_per_brand_per_market\n        type: string\n        primary_key: true\n        sql: brand_code||market_code||email\n      - meta:\n          ecto_field: first_name\n          ecto_field_type: string\n        name: given_name\n        type: string\n        description: Given Name\n        sql: first_name\n\n```\n\nThe dimensions and measures derive some defaults from `Ecto.Schema.field` properties.\nFor example the `dimension:` `type:` is derived from ecto if not given explicitly according to this rules:\n\nCube dimension types    | Ecto type               | Elixir type\n:---------------------- | :---------------------- | :---------------------\nnumber                  | :id                     | integer\nstring                  | :binary_id              | binary\nnumber, boolean         | :integer                | integer\nnumber, boolean enough? | :float                  | float\nboolean                 | :boolean                | boolean\nstring                  | UTF-8 encoded `string`  | string\nstring                  | :binary                 | :binary\nstring                  | :bitstring              | :bitstring\n`{:array, inner_type}`  | `list`                  | TODO geo?\nNot Supported now       | `:map`                  | `map`\nNot Supported now       | `{:map, inner_type}`    | `map`\nnumber                  | `:decimal`              | [`Decimal`](https://github.com/ericmj/decimal)\ntime                    | `:date`                 | `Date`\ntime                    | `:time`                 | `Time`\ntime                    | `:time_usec`            | `Time`\ntime                    | `:naive_datetime`       | `NaiveDateTime`\ntime                    | `:naive_datetime_usec`  | `NaiveDateTime`\ntime                    | `:utc_datetime`         | `DateTime`\ntime                    | `:utc_datetime_usec`    | `DateTime`\nnumber                  | `:duration`             | `Duration`\n\n\n\n\nThe goal of `PowerOfThree` is to cover 80% of cases where the `source` of Ecto Schema is a table and fields have real column names:\n _where *field name =:= database column name*_\n\nThe the support of all cube features is not the goal here.\nThe automation of obtaining the usable cube configs with minimal verbocity is: avoid typing more typos then needed.\n\nThe cube DSL allows the `sql:` - _any SQL_ query. If everyone can write SQL it does not mean everyone should.\nWriting good SQL is an art a few knew. In the memory of Patrick's Mother  the `PowerOfThree` will _not_ support `sql:`.\nWhile defining custom `sql:` may looks like an option, how would one validate the creative use of aliases in SQL?\nMeanwhile Ecto.Schema fields are available for references to define dimensions `type:`.","ref":"PowerOfThree.html#module-example"},{"type":"macro","title":"PowerOfThree.cube/3","doc":"","ref":"PowerOfThree.html#cube/3"},{"type":"macro","title":"PowerOfThree.dimension/2","doc":"Dimension first argument takes a single Ecto.Schema field or a list of Ecto.Schema fields.\n\nLets create a Dimension for several Ecto.Schema fields. A list of Ecto.Schema fields is mandatory.\nEcto.Schema fields concatenated into SQL: `brand_code||market_code||email`\nThe `primary_key: true` tells the cube how to distinguish unique records.","ref":"PowerOfThree.html#dimension/2"},{"type":"macro","title":"Examples - PowerOfThree.dimension/2","doc":"dimension(\n      [:brand_code, :market_code, :email],\n      name: :email_per_brand_per_market,\n      primary_key: true\n    )\n\n\nLets create a Dimension for a single Ecto.Schema field","ref":"PowerOfThree.html#dimension/2-examples"},{"type":"macro","title":"Examples - PowerOfThree.dimension/2","doc":"dimension(:brand_code, name: :brand, description: \"Beer\")","ref":"PowerOfThree.html#dimension/2-examples-1"},{"type":"macro","title":"PowerOfThree.measure/2","doc":"Measure first argument takes an atom `:count`, a single Ecto.Schema field or a list of Ecto.Schema fields.\n\n\nLets create a Measure for several Ecto.Schema fields. A list of Ecto.Schema fields reference and `:type` are mandatory.\nThe `sql:` is mandatory, must be valid SQL clause using the fields from list and returning a number.","ref":"PowerOfThree.html#measure/2"},{"type":"macro","title":"Examples - PowerOfThree.measure/2","doc":"measure([:tax_amount,:discount_total_amount],\n      sql: \"tax_amount + discount_total_amount\",\n      type: :sum,\n      description: \"two measures we want add together\"\n    )\n\n\nLets create a Measure for a single Ecto.Schema field\nThe Ecto.Schema field reference and `:type` are mandatory\nThe other cube measure DLS properties are passed through","ref":"PowerOfThree.html#measure/2-examples"},{"type":"macro","title":"Examples - PowerOfThree.measure/2","doc":"measure(:email,\n      name: :emails_distinct,\n      type: :count_distinct,\n      description: \"count distinct of emails\"\n    )\n\n\nLets create a Measure of type `:count`\nNo `:type` is needed\nThe other cube measure DLS properties are passed through","ref":"PowerOfThree.html#measure/2-examples-1"},{"type":"macro","title":"Examples - PowerOfThree.measure/2","doc":"measure(:count,\n      description: \"no need for fields for :count type measure\"\n    )","ref":"PowerOfThree.html#measure/2-examples-2"},{"type":"macro","title":"PowerOfThree.time_dimensions/1","doc":"Uses `:inserted_at` as default time dimension\ndefmacro cube(__CALLER__,cube_name, what_ecto_schema, block)\n\ndefp cube(caller,cube_name,what_ecto_schema, block) do","ref":"PowerOfThree.html#time_dimensions/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}